@@ -5,6 +5,7 @@ Provides functionality to query and control BAF i6 protocol devices.
 from __future__ import annotations
 
 import asyncio
+import copy
 import inspect
 import logging
 import time
@@ -73,7 +74,7 @@ class Device:
                 f"Invalid service: must have at least one address and a port: {service}"
             )
 
-        self.service = service
+        self._service = copy.deepcopy(service)
         self._query_interval_seconds = query_interval_seconds
 
         # Permanent Properties protobuf into which query results are merged.
@@ -85,6 +86,9 @@ class Device:
         self._dispatch_coro_callback_tasks: t.Set[asyncio.Task] = set()
 
         # Connection and periodic queries.
+        self._loop = asyncio.get_running_loop()
+        if self._loop is None:
+            raise RuntimeError("no running loop")
         self._run_fut: t.Optional[asyncio.Future] = None
         self._stop_requested = False
         self._next_connect_ts: float = time.monotonic()
@@ -119,20 +123,21 @@ class Device:
             string += f", Has Light: {self.has_light}"
         return string
 
-    def _maybe_prop(self, field: str) -> t.Optional[t.Any]:
-        return maybe_proto_field(t.cast(Message, self._properties), field)
+    @property
+    def service(self) -> Service:
+        return copy.deepcopy(self._service)
 
     @property
     def properties_dict(self) -> dict[str, t.Any]:
         """Return a dict created by merging the device's service and properties."""
         d = {
-            "dns_sd_uuid": self.service.uuid,
-            "service_name": self.service.service_name,
-            "name": self.service.device_name,
-            "model": self.service.model,
-            "api_version": self.service.api_version,
-            "ip_addresses": self.service.ip_addresses,
-            "port": self.service.port,
+            "dns_sd_uuid": self._service.uuid,
+            "service_name": self._service.service_name,
+            "name": self._service.device_name,
+            "model": self._service.model,
+            "api_version": self._service.api_version,
+            "ip_addresses": self._service.ip_addresses,
+            "port": self._service.port,
         }
         d.update(
             json_format.MessageToDict(
@@ -147,6 +152,8 @@ class Device:
         p.CopyFrom(self._properties)
         return p
 
+    # Client callbacks
+
     def add_callback(self, callback: t.Callable[[Device], None]) -> None:
         """Add a device update callback.
 
@@ -192,6 +199,11 @@ class Device:
             f"{self.name}: Dispatched {len(self._callbacks) + len(self._coro_callbacks)} client callbacks."
         )
 
+    # protoprop support
+
+    def _maybe_property(self, field: str) -> t.Optional[t.Any]:
+        return maybe_proto_field(t.cast(Message, self._properties), field)
+
     def _commit_property(self, p: aiobafi6_pb2.Properties) -> None:
         """Commit a property to the device.
 
@@ -211,6 +223,8 @@ class Device:
         _LOGGER.debug(f"{self.name}: Sending commit:\n{root}")
         self._transport.write(wireutils.serialize(root))
 
+    # Connection and query machinery
+
     def _sched_connect_or_signal_run_fut(self):
         """Schedules a `_connect` invocation or signals the run future.
 
@@ -230,7 +244,7 @@ class Device:
                 self._run_fut.set_result(None)
         else:
             _LOGGER.debug(f"{self.name}: Scheduling next connect invocation.")
-            self._connect_timer = asyncio.get_running_loop().call_at(
+            self._connect_timer = self._loop.call_at(
                 self._next_connect_ts,
                 self._connect,
             )
@@ -241,15 +255,17 @@ class Device:
             time.monotonic() + _DELAY_BETWEEN_CONNECT_ATTEMPTS_SECONDS
         )
         _LOGGER.debug(
-            f"{self.name}: Connecting to {self.service.ip_addresses[0]}:{self.service.port}."
+            f"{self.name}: Connecting to {self._service.ip_addresses[0]}:{self._service.port}."
         )
         connect_task = asyncio.create_task(
-            asyncio.get_running_loop().create_connection(
-                lambda: Protocol(self), self.service.ip_addresses[0], self.service.port
+            self._loop.create_connection(
+                lambda: Protocol(self),
+                self._service.ip_addresses[0],
+                self._service.port,
             )
         )
         connect_task.add_done_callback(self._finish_connect)
-        asyncio.get_running_loop().call_later(
+        self._loop.call_later(
             _DELAY_BETWEEN_CONNECT_ATTEMPTS_SECONDS, lambda: connect_task.cancel()
         )
         self._connect_task = connect_task
@@ -264,7 +280,7 @@ class Device:
             )
             self._transport = transport
             self._protocol = protocol
-            asyncio.get_running_loop().call_soon(self._query)
+            self._loop.call_soon(self._query)
         except (OSError, asyncio.CancelledError) as err:
             _LOGGER.debug(f"{self.name}: Connection failed: {err}")
             self._sched_connect_or_signal_run_fut()
@@ -311,7 +327,7 @@ class Device:
         _LOGGER.debug(f"{self.name}: Sending query:\n{root}")
         self._transport.write(wireutils.serialize(root))
         if self._query_interval_seconds > 0:
-            self._query_timer = asyncio.get_running_loop().call_later(
+            self._query_timer = self._loop.call_later(
                 self._query_interval_seconds, self._query
             )
 
@@ -324,7 +340,7 @@ class Device:
         Returns a future that will resolve when the device stops. Cancelling any future
         returned by this function will stop the device.
         """
-        fut = asyncio.get_running_loop().create_future()
+        fut = self._loop.create_future()
         if self._run_fut is None:
             self._start()
         assert self._run_fut is not None
@@ -359,7 +375,7 @@ class Device:
         assert self._run_fut is None
         assert not self._stop_requested
         _LOGGER.debug(f"{self.name}: Starting.")
-        self._run_fut = asyncio.get_running_loop().create_future()
+        self._run_fut = self._loop.create_future()
         self._run_fut.add_done_callback(self._finish_run)
         self._sched_connect_or_signal_run_fut()
 
@@ -396,6 +412,8 @@ class Device:
         self._run_fut = None
         self._stop_requested = False
 
+    # Availability
+
     @property
     def available(self) -> bool:
         """Return True when device is running and has values for critical properties."""
@@ -411,17 +429,20 @@ class Device:
     def name(self) -> str:
         if len(self._properties.name) > 0:
             return self._properties.name
-        if self.service.service_name is not None and len(self.service.service_name) > 0:
-            return self.service.service_name
+        if (
+            self._service.service_name is not None
+            and len(self._service.service_name) > 0
+        ):
+            return self._service.service_name
         if len(self._properties.mac_address) > 0:
             return self._properties.mac_address
-        return self.service.ip_addresses[0]
+        return self._service.ip_addresses[0]
 
     @property
     def model(self) -> t.Optional[str]:
         if len(self._properties.model) > 0:
             return self._properties.model
-        return self.service.model
+        return self._service.model
 
     firmware_version = ProtoProp[t.Optional[str]]()
     mac_address = ProtoProp[t.Optional[str]]()
@@ -432,7 +453,7 @@ class Device:
     def dns_sd_uuid(self) -> t.Optional[str]:
         if len(self._properties.dns_sd_uuid) > 0:
             return self._properties.dns_sd_uuid
-        return self.service.uuid
+        return self._service.uuid
 
     @property
     def has_fan(self) -> bool:
@@ -511,7 +532,7 @@ class Device:
     def ip_address(self) -> str:
         if len(self._properties.ip_address) > 0:
             return self._properties.ip_address
-        return self.service.ip_addresses[0]
+        return self._service.ip_addresses[0]
 
     @property
     def wifi_ssid(self) -> t.Optional[str]:
